////////////////////////////////////////////////////////////////////////////////
//                                 APICulture
//                        (c)  , 1999-2000
//    http://mazurkin.virtualave.net, mazurkin@mailru.com, mazurkin@chat.ru
// _____________________________________________________________________________
//                                   
////////////////////////////////////////////////////////////////////////////////

unit AclWindowMain;

interface

uses
  AclDialogMain, AclThreadCalculus,
  Windows, Messages, WinLite, ShellAPI;

const
  WM_TRAYICON = WM_USER + 1;

type
  TWindowMain = class(TLiteWindow)
  private
    FDialogMain : TDialogMain;
    FThreadCalc : TThreadCalculus;
    FMainMenu   : THandle;
    FTrayUp     : THandle;
    FTrayDown   : THandle;
    FTrayMenu   : THandle;
    FHotKeyAtom : THandle;
  protected
    procedure   CreateWindowParams(var WindowParams: TWindowParams); override;
  protected
    procedure   OnCreate(var Msg: TWMCreate); message WM_CREATE;
    procedure   OnDestroy(var Msg: TWMDestroy); message WM_DESTROY;
    procedure   OnSysCommand(var Msg: TWMSysCommand); message WM_SYSCOMMAND;
    procedure   OnMoving(var Msg: TMessage); message WM_MOVING;
    procedure   OnTray(var Msg: TMessage); message WM_TRAYICON;
    procedure   OnCommand(var Msg: TWMCommand); message WM_COMMAND;
    procedure   OnHotKey(var Msg: TWMHotKey); message WM_HOTKEY;
    procedure   OnEureka(var Msg: TMessage); message WM_EUREKA;
  protected
    procedure   OnCommandAbout;
    procedure   OnCommandOpenGL;
    procedure   OnCommandToggle;
    procedure   OnCommandSubclass;
    procedure   OnCommandExit;
  end;

implementation

uses AclDialogAbout, AclDialogOpenGL, AclDialogSubclass;

{$I res\resource.inc}

////////////////////////////////////////////////////////////////////////////////
// TWindowMain
// _____________________________________________________________________________
//  / 
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TWindowMain.CreateWindowParams(var WindowParams: TWindowParams);
begin
  inherited;
  //    
  WindowParams.Caption := 'APICulture';
  WindowParams.Style   := WS_CAPTION or WS_SYSMENU or WS_MINIMIZEBOX; 
  WindowParams.ExStyle := WS_EX_APPWINDOW or WS_EX_TOPMOST;
  WindowParams.WindowClass.hIcon := LoadIcon(hInstance, MAKEINTRESOURCE(ICON_WINDOW));
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TWindowMain.OnCreate(var Msg: TWMCreate);
var
  NID : TNotifyIconData;
  ClientRect : TRect;
begin
  //    
  FMainMenu := LoadMenu(hInstance, MAKEINTRESOURCE(MENUMAIN));
  SetMenu(WndHandle, FMainMenu);
  //       
  FDialogMain := TDialogMain.Create(WndHandle);
  //         
  SetWindowPos(FDialogMain.WndHandle, 0, 0, 0, 0, 0, SWP_NOSIZE+SWP_NOZORDER+SWP_SHOWWINDOW);
  //    
  GetWindowRect(FDialogMain.WndHandle, ClientRect);
  //       
  AdjustWindowRectEx(ClientRect, GetWindowLong(WndHandle, GWL_STYLE),
    True, GetWindowLong(WndHandle, GWL_EXSTYLE));
  //       
  SetWindowPos(WndHandle, HWND_TOP, 0, 0,
    ClientRect.Right-ClientRect.Left, ClientRect.Bottom-ClientRect.Top,
    SWP_NOMOVE+SWP_NOZORDER+SWP_SHOWWINDOW);
  //  
  FTrayUp   := LoadIcon(hInstance, MAKEINTRESOURCE(ICON_TRAYUP));
  FTrayDown := LoadIcon(hInstance, MAKEINTRESOURCE(ICON_TRAYDOWN));
  //    
  NID.cbSize := SizeOf(NID);
  NID.Wnd    := WndHandle;
  NID.uID    := 1;
  NID.uFlags := NIF_ICON+NIF_MESSAGE+NIF_TIP;
  NID.uCallbackMessage := WM_TRAYICON;
  NID.hIcon  := FTrayDown;
  NID.szTip  := 'APICulture';
  Shell_NotifyIcon(NIM_ADD, @NID);
  //   
  FTrayMenu := CreatePopupMenu;
  InsertMenu(FTrayMenu, 0, MF_BYPOSITION+MF_STRING, CMD_PROGRAM_TOGGLE, '&');
  InsertMenu(FTrayMenu, 1, MF_SEPARATOR, 0, nil);
  InsertMenu(FTrayMenu, 2, MF_BYPOSITION+MF_STRING, CMD_DIALOG_ABOUT, '& ...');
  InsertMenu(FTrayMenu, 3, MF_BYPOSITION+MF_STRING, CMD_PROGRAM_EXIT, '&');
  //     
  SetMenuDefaultItem(FTrayMenu, CMD_PROGRAM_TOGGLE, MF_BYCOMMAND);
  //   "" 
  FHotKeyAtom := GlobalAddAtom('APICulture HotKey');
  RegisterHotKey(WndHandle, FHotKeyAtom, MOD_CONTROL, $57); // VK_W = 0x57
  //    
  FThreadCalc := TThreadCalculus.CreateSlaved(WndHandle);
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TWindowMain.OnDestroy(var Msg: TWMDestroy);
var
  NID: TNotifyIconData;
begin
  //     -
  FThreadCalc.Terminate; 
  //  
  FThreadCalc.Free;
  //   "" 
  UnregisterHotKey(WndHandle, FHotKeyAtom);
  //   "" 
  GlobalDeleteAtom(FHotKeyAtom);
  //   
  DestroyMenu(FTrayMenu);
  //   
  DestroyMenu(FMainMenu);
  //    
  NID.cbSize := SizeOf(NID);
  NID.Wnd    := WndHandle;
  NID.uID    := 1;
  NID.uFlags := 0;
  Shell_NotifyIcon(NIM_DELETE, @NID);
  //  
  DestroyIcon(FTrayUp);
  DestroyIcon(FTrayDown);
  //   
  FDialogMain.Free;
end;

////////////////////////////////////////////////////////////////////////////////
// TWindowMain
// _____________________________________________________________________________
//    
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//     -   ""  
////////////////////////////////////////////////////////////////////////////////
procedure TWindowMain.OnMoving(var Msg: TMessage);
var
  WindowArea : PRect;
  WorkArea   : TRect;
  SenseX     : Integer;
  SenseY     : Integer;
begin
  //     
  WindowArea := PRect(Msg.LParam);
  //    
  SystemParametersInfo(SPI_GETWORKAREA, 0, @WorkArea, 0);
  //  
  SenseX := GetSystemMetrics(SM_CXVSCROLL) div 2;
  SenseY := GetSystemMetrics(SM_CYHSCROLL) div 2;
  //    
  if Abs(WindowArea.Left - WorkArea.Left) < SenseX then
    OffsetRect(WindowArea^, WorkArea.Left - WindowArea.Left, 0);
  //    
  if Abs(WindowArea.Right - WorkArea.Right) < SenseX then
    OffsetRect(WindowArea^, WorkArea.Right - WindowArea.Right, 0);
  //    
  if Abs(WindowArea.Top - WorkArea.Top) < SenseY then
    OffsetRect(WindowArea^, 0, WorkArea.Top - WindowArea.Top);
  //    
  if Abs(WindowArea.Bottom - WorkArea.Bottom) < SenseY then
    OffsetRect(WindowArea^, 0, WorkArea.Bottom - WindowArea.Bottom);
  // 
  Msg.Result := 1;
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TWindowMain.OnSysCommand(var Msg: TWMSysCommand);
begin
  //   
  case Msg.CmdType of
    SC_CLOSE :    //      ,   .
      begin
        OnCommandExit;
        Msg.Result := 0;
      end;
    SC_MINIMIZE : //     ,   
      begin
        OnCommandToggle;
        Msg.Result := 0;
      end;
    else          //     
      with TMessage(Msg) do Result := DefWindowProc(WndHandle, Msg, WParam, LParam);
  end;
end;

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TWindowMain.OnCommand(var Msg: TWMCommand);
begin
  //   
  case Msg.ItemID of
    CMD_DIALOG_ABOUT   : OnCommandAbout;
    CMD_DIALOG_OPENGL  : OnCommandOpenGL;
    CMD_DIALOG_SUBCLASS: OnCommandSubclass;
    CMD_PROGRAM_TOGGLE : OnCommandToggle;
    CMD_PROGRAM_EXIT   : OnCommandExit;
  end;
  // 
  Msg.Result := 0;
end;

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TWindowMain.OnTray(var Msg: TMessage);
var
  Point: TPoint;
begin
  case Msg.lParam of
    WM_LBUTTONDBLCLK:
      OnCommandToggle;
    WM_RBUTTONUP:
      begin
        //   
        GetCursorPos(Point);
        //   
        TrackPopupMenu(FTrayMenu, TPM_LEFTBUTTON, Point.X, Point.Y, 0, WndHandle, nil);
        //     
        PostMessage(WndHandle, WM_USER, 0, 0);
      end;
  end;
end;

////////////////////////////////////////////////////////////////////////////////
//   "" 
////////////////////////////////////////////////////////////////////////////////
procedure TWindowMain.OnHotKey(var Msg: TWMHotKey);
begin
  OnCommandToggle;
end;

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TWindowMain.OnEureka(var Msg: TMessage);
begin
  MessageBeep($FFFFFFFF);
end;

////////////////////////////////////////////////////////////////////////////////
// TWindowMain
// _____________________________________________________________________________
//     
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//   " "
////////////////////////////////////////////////////////////////////////////////
procedure TWindowMain.OnCommandAbout;
var
  Dialog : TDialogAbout;
begin
  Dialog := TDialogAbout.Create(WndHandle);
  Dialog.ShowModal;
  Dialog.Free;
end;

////////////////////////////////////////////////////////////////////////////////
//    OpenGL
////////////////////////////////////////////////////////////////////////////////
procedure TWindowMain.OnCommandOpenGL;
var
  Dialog : TDialogOpenGL;
begin
  Dialog := TDialogOpenGL.Create(WndHandle);
  Dialog.ShowModal;
  Dialog.Free;
end;

////////////////////////////////////////////////////////////////////////////////
//   subclass
////////////////////////////////////////////////////////////////////////////////
procedure TWindowMain.OnCommandSubclass;
var
  Dialog : TDialogSubclass;
begin
  Dialog := TDialogSubclass.Create(WndHandle);
  Dialog.ShowModal;
  Dialog.Free;
end;

////////////////////////////////////////////////////////////////////////////////
// / 
////////////////////////////////////////////////////////////////////////////////
procedure TWindowMain.OnCommandToggle;
var
  NID: TNotifyIconData;
begin
  //   
  if IsWindowVisible(WndHandle) then
  begin
    //  
    ShowWindow(WndHandle, SW_HIDE);
    ModifyMenu(FTrayMenu, CMD_PROGRAM_TOGGLE, MF_BYCOMMAND+MF_STRING, CMD_PROGRAM_TOGGLE, '&');
    NID.hIcon := FTrayUp;
  end else
  begin
    //  
    ShowWindow(WndHandle, SW_SHOW);
    SetForegroundWindow(WndHandle);
    ModifyMenu(FTrayMenu, CMD_PROGRAM_TOGGLE, MF_BYCOMMAND+MF_STRING, CMD_PROGRAM_TOGGLE, '&');
    NID.hIcon  := FTrayDown;
  end;
  //    
  NID.cbSize := SizeOf(NID);
  NID.Wnd    := WndHandle;
  NID.uID    := 1;
  NID.uFlags := NIF_ICON;
  Shell_NotifyIcon(NIM_MODIFY, @NID);
end;

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TWindowMain.OnCommandExit;
begin
  //   WM_QUIT
  PostQuitMessage(0);
end;

end.
